local addonName, vars = ...
local L = vars.L
GarajalAnnounce = LibStub("AceAddon-3.0"):NewAddon(addonName)
local addon = GarajalAnnounce 
vars.svnrev = {}
vars.svnrev["core.lua"] = tonumber(("$Revision: 10 $"):match("%d+"))

local defaults = {
  profile = {
    debug = true, -- for addon debugging
    enable = true,
    enablelfr = false,
    raidwarning = true,
    raidicons = true,
    whisper = true,
    healermana = true,
    minimap = {
        hide = false,
    },
    players = {},
    maxorder = 1,
  }
}

local players
local defaultpriority = 2

local spellids = {
  voodoo = 122151,  -- voodoo doll, ineligble for totem
  --voodoo  =  57723,  -- Exhaustion, for testing only
  crossed = 116161,  -- Crossed Over, in spirit realm up to 30 sec
  buffed  = 117549,  -- Spiritual innervation, 30 sec dps/mana buff
  frail   = 117723,  -- Frail Soul (heroic) cannot enter for 30 sec
  totem   = 116174,  -- boss summoned a new totem
}
local spellnames = {}
function addon:InitNames()
  for key, id in pairs(spellids) do
    if not spellnames[key] then
      spellnames[key] = GetSpellInfo(id)
    end
  end
end

local settings = defaults.profile
local optionsFrame
local charName
local minimapIcon = LibStub("LibDBIcon-1.0")
local LDB, LDBo

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage(addonName..": "..msg)
end
local function debug(msg)
  if addon.db.profile.debug then
     chatMsg(msg)
  end
end

local function UnitToName(unit)
  return GetUnitName(unit, true)
end

function addon:GetDiff()
  local instname, insttype, diff, diffname, maxPlayers, playerDifficulty, isDynamicInstance = GetInstanceInfo()
  addon.raidsize = maxPlayers
  if HasLFGRestrictions() or IsInLFGDungeon() then
     addon.raiddiff = "lfr"
  elseif diff == 5 or diff == 6 then
     addon.raiddiff = "heroic"
  else
     addon.raiddiff = "normal"
  end
end

function addon:EnabledDiff()
  if not settings.enable then 
    return false
  end
  addon:GetDiff()
  if addon.raiddiff == "lfr" then
    return settings.enablelfr
  else
    return true
  end
end

function addon:canAnnounce()
  return UnitIsGroupLeader("player") or UnitIsGroupAssistant("player")
end

function addon:ClearIcons()
  for i=1,40 do
    local unit = "raid"..i
    if UnitExists(unit) then
      SetRaidTarget(unit, 0)
    end
  end
end

function addon:InZone()
  local oldmap = GetCurrentMapAreaID()
  local oldlvl = GetCurrentMapDungeonLevel()
  SetMapToCurrentZone()
  local map = GetCurrentMapAreaID()
  local lvl = GetCurrentMapDungeonLevel()
  local inzone = (map == 896 and lvl == 2)
  local unitX, unitY = GetPlayerMapPosition("player")
  if not unitX then unitX,unitY = 0,0 end
  unitX, unitY = unitX*100, unitY*100
  SetMapByID(oldmap)
  if lvl and lvl > 0 then
    SetDungeonMapLevel(oldlvl)
  end
  return inzone, unitX, unitY
end

local combat_events = {
 "UNIT_SPELLCAST_SUCCEEDED",
 "PLAYER_REGEN_DISABLED",
 "UNIT_AURA",
 -- "COMBAT_LOG_EVENT_UNFILTERED"
}

function addon:CheckEnable()
  if not addon.init then return end
  --debug("CheckEnable")
  if addon:InZone() and addon:EnabledDiff() and not addon:canAnnounce() and not addon.offwarn then
    chatMsg(L["You must have raid lead or assist to make annoucements."])
    addon.offwarn = true
  end
  if addon:InZone() and ((addon:canAnnounce() and addon:EnabledDiff()) or addon.db.profile.debug) then
    addon:InitNames()
    addon:ScanRaid()
    if addon.enabled then return end
    addon.enabled = true
    addon.offwarn = false
    addon:ScanRaid(nil,true) -- clean players not present
    chatMsg(L["Version"].." "..addon.version)
    chatMsg(L["Addon enabled. Type '/ga config' for options."])
    for _,evt in pairs(combat_events) do 
      addon:RegisterEvent(evt)
    end
  else -- wrong zone
    if not addon.enabled then return end
    chatMsg(L["Addon disabled."])
    addon.enabled = false
    for _,evt in pairs(combat_events) do 
      addon:UnregisterEvent(evt)
    end
  end

end

function addon:ScanRaid(reset, cleanold)
  if InCombatLockdown() then return end
  addon:GetDiff()
  local cnt = 0
  if reset then
    settings.maxorder = 0
    wipe(players)
  end
  for _,info in pairs(players) do
    info.unit = nil
  end
  addon.missingroles = 0
  for i=1,40 do
    local unit = "raid"..i
    if UnitExists(unit) then
      cnt = cnt + 1
      local name = UnitToName(unit)
      local info = players[name] or {}
      players[name] = info
      info.unit = unit
      info.name = name
      info.role = UnitGroupRolesAssigned(unit)
      if info.role == "NONE" then
        addon.missingroles = addon.missingroles + 1
      end
      info.priority = (reset and defaultpriority) or info.priority or defaultpriority
      if info.priority < 1 or info.priority > 3 then
         info.priority = defaultpriority
      end
      if not info.order then -- reset or new player
        info.order = settings.maxorder
        settings.maxorder = settings.maxorder + 1
      end
      info.lastaction = 0
      --debug(name..":"..info.role..":"..info.priority..":"..info.order)
    end
  end
  if addon.missingroles > 0 then
    chatMsg(string.format(L["WARNING: %d players do not have roles assigned and will be ignored."],addon.missingroles))
  end
  if cleanold then -- remove stored players with default settings not in current raid, to prevent SV bloat
    for name,info in pairs(players) do
      if not info.unit and (info.priority == defaultpriority) then
        debug("Cleaning old player: "..name)
        players[name] = nil
      end
    end
  end
  debug("Processed "..cnt.." players, "..addon.missingroles.." unassigned roles")
  addon.scanned = true
end

local function playerSort(i1, i2)
  if (i1.delay ~= i2.delay) then 
    return i1.delay < i2.delay
  elseif (settings.healermana and i1.role == "HEALER" and i1.mana ~= i2.mana) then
    return i1.mana < i2.mana
  elseif (i1.priority ~= i2.priority) then
    return i1.priority < i2.priority
  elseif (i1.lastaction ~= i2.lastaction) then
    return i1.lastaction < i2.lastaction
  else
    return i1.order <= i2.order
  end
end

local healerElig = {}
local dpsElig = {}
local playersChosen = {}
function addon:handleTotem()
  local now = GetTime()
  addon.lasttotem = addon.lasttotem or 0
  if now < addon.lasttotem + 5 then
    debug("Ignoring duplicate handleTotem")
    return
  end
  addon.lasttotem = now
  addon.totemcount = (addon.totemcount or 0) + 1
  debug("handleTotem() "..addon.totemcount);
  --debug(GetRealZoneText()..":"..GetSubZoneText()..":"..GetZoneText()..":"..GetMinimapZoneText())
  wipe(healerElig)
  wipe(dpsElig)
  for i=1,40 do
    local unit = "raid"..i
    local zone, online, isDead = select(7,GetRaidRosterInfo(i))
    if UnitExists(unit) and (addon.dryrun or 
       (online and not isDead and zone == addon.zone and
        not UnitIsDeadOrGhost(unit) and UnitIsConnected(unit))) then
       local name = UnitToName(unit)
       local info = players[name]
       local list = nil
       if info and info.role == "DAMAGER" then 
         list = dpsElig
       elseif info and info.role == "HEALER" then 
         list = healerElig
         info.mana = UnitPower(unit, SPELL_POWER_MANA)/UnitPowerMax(unit, SPELL_POWER_MANA)
       end
       local elig = list
       while elig do -- fake loop to simulate continue
         local expires
         if UnitDebuff(unit, spellnames.voodoo) then
	   debug(name.." HAS VOODOO")
	   elig = false; break;
	 end
         info.delay = 0
	 expires = select(7,UnitDebuff(unit, spellnames.crossed))
	 if expires then
	   debug(name.." IS CROSSED")
	   if addon.raiddiff == "heroic" then
             elig = false; break; -- 30 sec lockout once they leave
	   else
	     info.delay = math.max(info.delay, (expires - now))    
	   end
	 end
	 expires = select(7,UnitBuff(unit, spellnames.buffed))
	 if expires and info.role == "DAMAGER" then
	   debug(name.." HAS BUFF")
	   info.delay = math.max(info.delay, (expires - now))    
	 end
	 if addon.raiddiff == "heroic" then
	   expires = select(7,UnitDebuff(unit, spellnames.frail))
	   if expires then
	     debug(name.." IS FRAIL")
	     info.delay = math.max(info.delay, (expires - now))    
	   end
	 end
         debug("Elig "..(info.role or "nil")..": "..(name or "nil").."  delay="..(info.delay or "nil").." lastaction="..(info.lastaction or 0))
         table.insert(list,info)
	 break;
       end -- fake loop
    end -- unit alive
  end  -- raid members
  table.sort(healerElig, playerSort)
  table.sort(dpsElig, playerSort)
  local healercnt = 1
  local dpscnt = 2
  local rtmod = 6
  if addon.raidsize > 10 then
    dpscnt = 3
    rtmod = 8
  end
  wipe(playersChosen)
  for i=1,healercnt do
    if healerElig[i] then
      table.insert(playersChosen, healerElig[i])
    else
      chatMsg(L["WARNING: Not enough eligible healers!"])
    end
  end
  for i=1,dpscnt do
    if dpsElig[i] then
      table.insert(playersChosen, dpsElig[i])
    else
      chatMsg(L["WARNING: Not enough eligible dps!"])
    end
  end
  local msg = L["Spirit Totem"].." " .. addon.totemcount..":"
  local delay = 0
  addon.rt = addon.rt or 0
  for idx,info in ipairs(playersChosen) do
    delay = math.max(delay, info.delay)
    info.lastaction = now
    local rt = addon.rt + 1
    addon.rt = (addon.rt + 1) % rtmod
    msg = msg .. " "
    if settings.raidicons then
      if addon:canAnnounce() and not addon.suppress then
        SetRaidTarget(info.unit, rt)
      end
      msg = msg .. "{rt"..rt.."}"     
    end
    msg = msg .. info.name
    if addon:canAnnounce() and settings.whisper and not addon.suppress then
      SendChatMessage(L["Run to the totem!!!"], "WHISPER", nil, info.name)
    end
  end
  if delay > 1 then
    msg = " ("..L["DELAY"]..SecondsToTime(delay)..")"
  end
  debug(msg)
  if addon:canAnnounce() and settings.raidwarning and not addon.suppress then
    SendChatMessage(msg, "RAID_WARNING")
  end
end

function addon:PLAYER_REGEN_DISABLED() -- entering combat
  addon.totemcount = 0
  addon.zone = GetRealZoneText()
  if settings.raidicons and addon:canAnnounce() and not addon.suppress then
    addon:ClearIcons()
  end
end

function addon:UNIT_AURA(unit) -- detect actual cross-overs
  local expires = select(7,UnitDebuff(unit, spellnames.crossed))
  if expires then -- has crossed aura
    local now = GetTime()
    if expires > now + 25 then -- just crossed
       local name = UnitToName(unit)
       local info = players[name]
       if info then
          info.lastaction = now
       end
    end
  end
  if settings.raidicons and GetRaidTargetIndex(unit) and UnitBuff(unit, spellnames.buffed) then
     SetRaidTarget(unit, 0) -- got buffed, remove icon
  end
end

function addon:COMBAT_LOG_EVENT_UNFILTERED(...)
  local event,_,_,_,_,_,_,_,_,_,id = ...
  if event == "SPELL_CAST_SUCCESS" and id == spellids.totem then
    addon:handleTotem()
  end
end

function addon:UNIT_SPELLCAST_SUCCEEDED(event, unitid, spellName, _, _, spellID)
  if spellID == spellids.totem then
    addon:handleTotem()
  end 
end

function addon:BuildOptions() 
addon.options = {
  type = "group",
  set = function(info,val)
          local s = settings ; for i = 2,#info-1 do s = s[info[i]] end
          s[info[#info]] = val; debug(info[#info].." set to: "..tostring(val))
          addon:Update()
        end,
  get = function(info)
          local s = settings ; for i = 2,#info-1 do s = s[info[i]] end
          return s[info[#info]] end,
  args = {
   general = {
    type = "group",
    inline = true,
    name = L["General"],
    args = {
      debug = {
        name = L["Debug"],
        desc = L["Toggle debugging output"],
        type = "toggle",
        guiHidden = true,
      },
      test = {
        name = "Test",
        desc = "Totem test",
        type = "execute",
        guiHidden = true,
        cmdHidden = true,
        func = function() 
	  addon.dryrun = true
	  if not addon.scanned then addon:ScanRaid(); end; 
	  addon:handleTotem() 
	  addon.dryrun = false
	end,
      },
      clear = {
        name = "Clear",
        desc = "Clear test",
        type = "execute",
        guiHidden = true,
        cmdHidden = true,
        func = function() addon:ClearIcons(); addon:ScanRaid(); end,
      },
      config = {
        name = L["Config"],
        desc = L["Open the configuration GUI"],
        type = "execute",
        guiHidden = true,
        func = function() addon:Config() end,
      },
      -- -----------------------------------------------------------------------------------------------
      version = {
        order = 1,
	type = "description",
        fontSize = "medium",
	name = L["Version"].." "..addon.version.."\n",
      },
        
      minimap = {
        order = 15,
        name = L["Minimap Icon"],
        desc = L["Display minimap icon"],
        type = "toggle",
        set = function(info,val)
          settings.minimap.hide = not val
          addon:Update()
	end,
        get = function() return not settings.minimap.hide end,
      },
      enable = {
        order = 50,
	name = L["Enable"],
	desc = L["Enable the addon"],
        type = "toggle",
      },
      enablelfr = {
        order = 50,
	name = L["Enable in LFR"],
	desc = L["Enable the addon in Looking For Raid difficulty"],
        type = "toggle",
	disabled = function() return not settings.enable end,
      },
      -- -----------------------------------------------------------------------------------------------
      aheader = {
        name = L["Announcement Options"],
        type = "header",
        cmdHidden = true,
        order = 99,
      },
      announcewarn = {
        order = 99.9,
	type = "description",
        fontSize = "medium",
	name = function() 
	  if addon:canAnnounce() then
	    return ""
	  else
	    return "\124cffff0000".. L["You must have raid lead or assist to make annoucements."].."\124r"
	  end
	end,
      },
      raidwarning = {
        order = 100,
        name = L["Raid Warning"],
        desc = L["Send a raid warning listing selected players"],
        type = "toggle",
	disabled = function() return not addon:EnabledDiff() end,
      },
      whisper = {
        order = 110,
        name = L["Whisper"],
        desc = L["Whisper selected players"],
        type = "toggle",
	disabled = function() return not addon:EnabledDiff() end,
      },
      raidicons = {
        order = 120,
        name = L["Raid Icons"],
        desc = L["Set raid icons on selected players"],
        type = "toggle",
	disabled = function() return not addon:EnabledDiff() end,
      },
      -- -----------------------------------------------------------------------------------------------
      rheader = {
        name = L["Raid"],
        type = "header",
        cmdHidden = true,
        order = 199,
      },
      rolecheck = {
        order = 200,
        name = L["Role Check"],
        desc = L["Start a Raid Role Check"],
        type = "execute",
	disabled = function() return not addon:canAnnounce() end,
        func = function() InitiateRolePoll() end,
      },
      refresh = {
        order = 205,
        name = L["Refresh"],
        desc = L["Refresh this display"],
        type = "execute",
        func = function() addon:RefreshConfigPanel() end,
        cmdHidden = true,
      },
      reset = {
        order = 209,
        name = L["Reset"],
        desc = L["Reset player settings"],
        type = "execute",
        func = function() addon:ScanRaid(true); addon:RefreshConfigPanel() end,
      },
      rolecheckwarn = {
        order = 210,
	type = "description",
        fontSize = "medium",
	name = function() 
	  if addon.missingroles and addon.missingroles > 0 then 
	    return string.format("\124cffff0000"..L["WARNING: %d players do not have roles assigned and will be ignored."].."\124r",addon.missingroles)
	  else
	    return ""
	  end
	end,
      },
      -- -----------------------------------------------------------------------------------------------
      healer = {
        order = 300,
        name = function() -- this filthy hack is brought to you by the overly restrictive AceConfig group API
          addon.options.args.general.args.healer.args = addon:playerconfig(true)
          return L["Healers"] 
        end,
        --inline = true,
        type = "group",
        args = {},
      },
      -- -----------------------------------------------------------------------------------------------
      dps = {
        order = 310,
        name = function() -- this filthy hack is brought to you by the overly restrictive AceConfig group API
          addon.options.args.general.args.dps.args = addon:playerconfig(false)
          return L["DPS"] 
        end,
        --inline = true,
        type = "group",
        args = {},
      },
      -- -----------------------------------------------------------------------------------------------
    },
  }
 }
} 
end

function addon:playerconfig(healer)
  local ret = {
  --[[
      list = {
            order = 100,
            type = "group",
	    inline = false,
	    childGroups = "tree",
            name = PLAYER,
            args = { },
      },
      --]]
  }
  if healer then
    ret.mana = {
       	order = 50,
	name = L["Prioritize by mana"],
	desc = L["Prioritize eligible healers with lowest mana"],
        type = "toggle",
  	set = function(info,val)
	  settings.healermana = val
          debug("healermana set to: "..tostring(val))
          addon:Update()
        end,
        get = function(info) return settings.healermana end,
    }
  end
  for name,info in pairs(players) do
    if info.unit and 
       ((healer and (info.role == "HEALER")) or 
        (not healer and (info.role == "DAMAGER"))) then
      local order = info.order + 10000
      local suffix = ""
      if healer and settings.healermana then
          -- nothing
      elseif info.priority == 1 then
          order = order - 10000;
          suffix = " ("..L["High Priority"]..")"
      elseif info.priority == 3 then
          order = order + 10000;
          suffix = " ("..L["Backup"]..")"
      end
      ret[name] = {
          order = order,
          type = "group",
          name = name..suffix,
	  args = {
            priority = {
	      order = 100,
	      name = L["Priority Group"]..": "..name,
	      type = "select",
	      values = { [1] = L["High Priority"], [2] = L["Default"], [3] = L["Backup"] },
	      set = function(i,val) info.priority = val; addon:RefreshConfigPanel() end,
	      get = function(i) return info.priority end,
	      disabled = healer and settings.healermana,
            },
	    prioritydesc = {
	      order = 110,
	      type = "description",
              fontSize = "medium",
	      name = L["High priority players are assigned to the totem whenever possible. Backup players are only sent when no others are available."],
	    },
	    orderheader = {
	      order = 200,
	      name = L["Initial order within priority"],
	      type = "header",
	    },
	    orderdesc = {
	      order = 205,
	      type = "description",
              fontSize = "medium",
	      name = L["Ordering within a priority group defines the approximate rotation in which players are considered for totem assignment, subject to debuff constraints."],
	    },
	    orderup = {
	      order = 210,
	      type = "execute",
	      name = L["Up"],
	      disabled = (healer and settings.healermana) or not addon:orderpeer(info,true),
	      func = function() addon:orderswap(info,true); addon:RefreshConfigPanel() end,
	    },
	    orderdown = {
	      order = 220,
	      type = "execute",
	      name = L["Down"],
	      disabled = (healer and settings.healermana) or not addon:orderpeer(info,false),
	      func = function() addon:orderswap(info,false); addon:RefreshConfigPanel() end,
	    },
          },
      }
    end
  end
  return ret
end

function addon:orderpeer(info, up) -- return the closest ordered peer in raid in the given direction within same priority group
  local peer = nil
  for _,pi in pairs(players) do
    if pi ~= info and pi.unit and pi.priority == info.priority and pi.role == info.role then
      if (up     and pi.order < info.order and (not peer or pi.order > peer.order)) or 
         (not up and pi.order > info.order and (not peer or pi.order < peer.order)) then
	   peer = pi
      end
    end
  end
  return peer
end

function addon:orderswap(info, up) -- swap order with the closest peer in raid in the given direction
  local peer = addon:orderpeer(info, up)
  if not peer then return end
  local tmp = peer.order
  peer.order = info.order
  info.order = tmp
end

local function table_clone(t)
  if not t then return nil
  elseif type(t) == "table" then
    local res = {}
    for k,v in pairs(t) do
      res[table_clone(k)] = table_clone(v)
    end
    return res
  else
    return t
  end
end

function addon:RefreshConfig()
  -- things to do after load or settings are reset
  debug("RefreshConfig")
  settings = addon.db.profile
  addon.settings = settings
  charName = UnitName("player")
  for k,v in pairs(defaults.profile) do
     if settings[k] == nil then
       settings[k] = table_clone(v)
     end
  end
  players = settings.players
  addon.players = players
  settings.loaded = true
  addon:Update()
end

function addon:Update()
  -- things to do when settings change
  if LDBo then
    if settings.minimap.hide then
      minimapIcon:Hide(addonName)
    else
      minimapIcon:Show(addonName)
    end
  end
  addon:CheckEnable()
end

function addon:SetupVersion()
   local svnrev = 0
   local files = vars.svnrev
   files["X-Build"] = tonumber((GetAddOnMetadata(addonName, "X-Build") or ""):match("%d+"))
   files["X-Revision"] = tonumber((GetAddOnMetadata(addonName, "X-Revision") or ""):match("%d+"))
   for _,v in pairs(files) do -- determine highest file revision
     if v and v > svnrev then
       svnrev = v
     end
   end
   addon.revision = svnrev

   files["X-Curse-Packaged-Version"] = GetAddOnMetadata(addonName, "X-Curse-Packaged-Version")
   files["Version"] = GetAddOnMetadata(addonName, "Version")
   addon.version = files["X-Curse-Packaged-Version"] or files["Version"] or "@"
   if string.find(addon.version, "@") then -- dev copy uses "@.project-version.@"
      addon.version = "r"..svnrev
   end
end


function addon:OnInitialize()
  addon:SetupVersion()
  addon.db = LibStub("AceDB-3.0"):New("GarajalAnnounceDB", defaults)
  addon:RefreshConfig()
  addon:BuildOptions()
  LibStub("AceEvent-3.0"):Embed(addon)
  LibStub("AceConfigRegistry-3.0"):ValidateOptionsTable(addon.options, addonName)
  LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, addon.options, {"garajalannounce", "ga"})
  optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, addonName, nil, "general")
  optionsFrame.default = function()
       for k,v in pairs(defaults.profile) do settings[k] = table_clone(v) end
       addon:RefreshConfig()
       if InterfaceOptionsFrame:IsShown() then
         addon:Config(); addon:Config()
       end
  end
  addon.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(addon.db)
  LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, L["Profiles"], addonName, "profiles")

  debug("OnInitialize")

  self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
  self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
  self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
  self.db.RegisterCallback(self, "OnDatabaseReset", "RefreshConfig")
  addon:InitNames()
  addon.init = true
end

function addon:Config()
  addon:ScanRaid()
  if optionsFrame then
    if ( InterfaceOptionsFrame:IsShown() ) then
      InterfaceOptionsFrame:Hide();
    else
      InterfaceOptionsFrame_OpenToCategory(optionsFrame)
    end
  end
end

function addon:RefreshConfigPanel()
  addon:ScanRaid()
  if optionsFrame then
    if ( InterfaceOptionsFrame:IsShown() ) then
      InterfaceOptionsFrame:Hide();
      InterfaceOptionsFrame_OpenToCategory(optionsFrame)
    end
  end
end

function addon:OnEnable()
  debug("OnEnable")
  self:RegisterEvent("GROUP_ROSTER_UPDATE", "CheckEnable")
  self:RegisterEvent("ZONE_CHANGED","CheckEnable")
  self:RegisterEvent("ZONE_CHANGED_INDOORS","CheckEnable")
  self:RegisterEvent("ZONE_CHANGED_NEW_AREA","CheckEnable")
 
  if LDB then
    return
  end
  if AceLibrary and AceLibrary:HasInstance("LibDataBroker-1.1") then
    LDB = AceLibrary("LibDataBroker-1.1")
  elseif LibStub then
    LDB = LibStub:GetLibrary("LibDataBroker-1.1",true)
  end
  if LDB then
    LDBo = LDB:NewDataObject(addonName, {
        type = "launcher",
        label = addonName,
        icon = "Interface\\Icons\\spell_nature_agitatingtotem",
        OnClick = function(self, button)
                if button == "RightButton" then
                        addon:Config()
                else
                        addon:Config()
                end
        end,
        OnTooltipShow = function(tooltip)
                if tooltip and tooltip.AddLine then
                        tooltip:SetText(addonName)
                        --tooltip:AddLine("|cffff8040"..L["Left Click"].."|r "..L["to toggle status window"])
                        tooltip:AddLine("|cffff8040"..L["Right Click"].."|r "..L["for options"])
                        tooltip:Show()
                end
        end,
     })
  end 

  if LDBo then
    minimapIcon:Register(addonName, LDBo, settings.minimap)
  end
  addon:Update()
end



